@@ -38,9 +38,10 @@ class Identifier(object):
     :type name: string
     :param name: The name of the identifier
     """
-    def __init__(self, name):
+    def __init__(self, name, member_name=None):
         #: (``string``) The name of the identifier
         self.name = name
+        self.member_name = member_name
 
 
 class Action(object):
@@ -72,7 +73,6 @@ class Action(object):
         self.path = definition.get('path')
 
 
-
 class DefinitionWithParams(object):
     """
     An item which has parameters exposed via the ``params`` property.
@@ -428,7 +428,10 @@ class ResourceModel(object):
 
         for item in self._definition.get('identifiers', []):
             name = self._get_name('identifier', item['name'])
-            identifiers.append(Identifier(name))
+            member_name = item.get('memberName', None)
+            if member_name:
+                member_name = self._get_name('attribute', member_name)
+            identifiers.append(Identifier(name, member_name))
 
         return identifiers
 
