@@ -32,3 +32,17 @@ class TestSession(unittest.TestCase):
         self.session.events.emit('myevent', my_list=initial_list)
         # Ensure that the registered handler was called.
         self.assertEqual(initial_list, ['my_handler called'])
+
+    def test_can_access_region_property(self):
+        session = boto3.session.Session(region_name='us-west-1')
+        self.assertEqual(session.region_name, 'us-west-1')
+
+    def test_get_available_partitions(self):
+        partitions = self.session.get_available_partitions()
+        self.assertIsInstance(partitions, list)
+        self.assertTrue(partitions)
+
+    def test_get_available_regions(self):
+        regions = self.session.get_available_regions('s3')
+        self.assertIsInstance(regions, list)
+        self.assertTrue(regions)
