@@ -411,7 +411,7 @@ class TestResourceFactory(BaseTestResourceFactory):
         # Accessing a property should call load
         self.assertEqual(resource.e_tag, 'tag',
             'ETag property returned wrong value')
-        action.assert_called_once()
+        self.assertEqual(action.call_count, 1)
 
         # Both params should have been loaded into the data bag
         self.assertIn('ETag', resource.meta.data)
@@ -421,7 +421,7 @@ class TestResourceFactory(BaseTestResourceFactory):
         # instead of making a second call.
         self.assertEqual(resource.last_modified, 'never',
             'LastModified property returned wrong value')
-        action.assert_called_once()
+        self.assertEqual(action.call_count, 1)
 
     @mock.patch('boto3.resources.factory.ServiceAction')
     def test_resource_lazy_properties_missing_load(self, action_cls):
@@ -456,6 +456,35 @@ class TestResourceFactory(BaseTestResourceFactory):
         with self.assertRaises(ResourceLoadException):
             resource.last_modified
 
+    @mock.patch('boto3.resources.factory.ServiceAction')
+    def test_resource_aliases_identifiers(self, action_cls):
+        model = {
+            'shape': 'TestShape',
+            'identifiers': [
+                {'name': 'id', 'memberName': 'foo_id'}
+            ]
+        }
+        shape = DenormalizedStructureBuilder().with_members({
+            'foo_id': {
+                'type': 'string',
+            },
+            'bar': {
+                'type': 'string'
+            },
+        }).build_model()
+        service_model = mock.Mock()
+        service_model.shape_for.return_value = shape
+
+        shape_id = 'baz'
+        resource = self.load(
+            'test', model, service_model=service_model)(shape_id)
+
+        try:
+            self.assertEqual(resource.id, shape_id)
+            self.assertEqual(resource.foo_id, shape_id)
+        except ResourceLoadException:
+            self.fail("Load attempted on identifier alias.")
+
     def test_resource_loads_references(self):
         model = {
             'shape': 'InstanceShape',
@@ -827,6 +856,13 @@ class TestServiceResourceSubresources(BaseTestResourceFactory):
         self.assertIn('PriorityQueue', dir(resource))
         self.assertIn('Message', dir(resource))
 
+    def test_get_available_subresources(self):
+        resource = self.load('test', self.model, self.defs)()
+        self.assertTrue(hasattr(resource, 'get_available_subresources'))
+        subresources = sorted(resource.get_available_subresources())
+        expected = sorted(['PriorityQueue', 'Message', 'QueueObject'])
+        self.assertEqual(subresources, expected)
+
     def test_subresource_missing_all_subresources(self):
         resource = self.load('test', self.model, self.defs)()
         message = resource.Message('url', 'handle')
@@ -846,8 +882,9 @@ class TestServiceResourceSubresources(BaseTestResourceFactory):
 
         # Verify we send out the class attributes dict.
         actual_class_attrs = sorted(call_args[1]['class_attributes'])
-        self.assertEqual(actual_class_attrs,
-                         ['Message', 'PriorityQueue', 'QueueObject', 'meta'])
+        self.assertEqual(actual_class_attrs, [
+            'Message', 'PriorityQueue', 'QueueObject',
+            'get_available_subresources', 'meta'])
 
         base_classes = sorted(call_args[1]['base_classes'])
         self.assertEqual(base_classes, [ServiceResource])
